@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011-2018 Mellanox Technologies Ltd.
+ * Copyright (c) 2011-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification,
@@ -31,14 +31,14 @@
 
 #define __STDC_FORMAT_MACROS
 
-#ifdef WIN32
+#ifdef __windows__
 #include <WS2tcpip.h>
-#include <sys/types.h>
+#include <Winsock2.h>
 #include <unordered_map>
 #include <Winbase.h>
 #include <stdint.h>
-
-typedef uint16_t in_port_t;
+#include <afunix.h>
+typedef unsigned short int sa_family_t;
 
 #else
 
@@ -60,10 +60,8 @@ typedef uint16_t in_port_t;
 
 #include <stdbool.h>
 #include <stdint.h>
-#include <tr1/unordered_map>
+#include <unordered_map>
 #include <unistd.h>   /* getopt() and sleep()*/
-#include <inttypes.h> /* printf PRItn */
-#include <regex.h>
 #include <poll.h>
 #include <pthread.h>
 #include <sys/time.h>   /* timers*/
@@ -72,12 +70,13 @@ typedef uint16_t in_port_t;
 #include <sys/syscall.h>
 #include <arpa/inet.h>   /* internet address manipulation */
 #include <netinet/in.h>  /* internet address manipulation */
-#include <netdb.h>       /* gethostbyname() */
+#include <netdb.h>       /* getaddrinfo() */
 #include <netinet/tcp.h> /* tcp specific */
 #include <sys/resource.h>
 
 #endif
 
+#include <inttypes.h> /* printf PRItn */
 #include <stdlib.h> /* random()*/
 #include <string.h>
 #include <stdio.h>
@@ -95,13 +94,53 @@ typedef uint16_t in_port_t;
 #include "ticks.h"
 #include "message.h"
 #include "playback.h"
+#include "ip_address.h"
 
-#if !defined(WIN32) && !defined(__FreeBSD__)
-#include "vma-redirect.h"
-#ifdef USING_VMA_EXTRA_API
+#if !defined(__windows__) && !defined(__FreeBSD__)
+#include "vma-xlio-redirect.h"
+#ifdef USING_VMA_EXTRA_API // VMA
+#define RING_LOGIC_PER_INTERFACE VMA_RING_LOGIC_PER_INTERFACE
+#define RING_LOGIC_PER_IP VMA_RING_LOGIC_PER_IP
+#define RING_LOGIC_PER_SOCKET VMA_RING_LOGIC_PER_SOCKET
+#define RING_LOGIC_PER_USER_ID VMA_RING_LOGIC_PER_USER_ID
+#define RING_LOGIC_PER_THREAD VMA_RING_LOGIC_PER_THREAD
+#define RING_LOGIC_PER_CORE VMA_RING_LOGIC_PER_CORE
+#define RING_LOGIC_PER_CORE_ATTACH_THREADS VMA_RING_LOGIC_PER_CORE_ATTACH_THREADS
+#define RING_LOGIC_LAST VMA_RING_LOGIC_LAST
+#define ring_logic_t vma_ring_logic_t
 #include <mellanox/vma_extra.h>
-#endif
-#endif
+#undef RING_LOGIC_PER_INTERFACE
+#undef RING_LOGIC_PER_IP
+#undef RING_LOGIC_PER_SOCKET
+#undef RING_LOGIC_PER_USER_ID
+#undef RING_LOGIC_PER_THREAD
+#undef RING_LOGIC_PER_CORE
+#undef RING_LOGIC_PER_CORE_ATTACH_THREADS
+#undef RING_LOGIC_LAST
+#undef ring_logic_t
+#endif // USING_VMA_EXTRA_API
+#ifdef USING_XLIO_EXTRA_API // XLIO
+#define RING_LOGIC_PER_INTERFACE XLIO_RING_LOGIC_PER_INTERFACE
+#define RING_LOGIC_PER_IP XLIO_RING_LOGIC_PER_IP
+#define RING_LOGIC_PER_SOCKET XLIO_RING_LOGIC_PER_SOCKET
+#define RING_LOGIC_PER_USER_ID XLIO_RING_LOGIC_PER_USER_ID
+#define RING_LOGIC_PER_THREAD XLIO_RING_LOGIC_PER_THREAD
+#define RING_LOGIC_PER_CORE XLIO_RING_LOGIC_PER_CORE
+#define RING_LOGIC_PER_CORE_ATTACH_THREADS XLIO_RING_LOGIC_PER_CORE_ATTACH_THREADS
+#define RING_LOGIC_LAST XLIO_RING_LOGIC_LAST
+#define ring_logic_t xlio_ring_logic_t
+#include <mellanox/xlio_extra.h>
+#undef RING_LOGIC_PER_INTERFACE
+#undef RING_LOGIC_PER_IP
+#undef RING_LOGIC_PER_SOCKET
+#undef RING_LOGIC_PER_USER_ID
+#undef RING_LOGIC_PER_THREAD
+#undef RING_LOGIC_PER_CORE
+#undef RING_LOGIC_PER_CORE_ATTACH_THREADS
+#undef RING_LOGIC_LAST
+#undef ring_logic_t
+#endif // USING_XLIO_EXTRA_API
+#endif // !WIN32 && !__FreeBSD__
 
 #define MIN_PAYLOAD_SIZE (MsgHeader::EFFECTIVE_SIZE)
 extern int MAX_PAYLOAD_SIZE;
@@ -119,6 +158,8 @@ const uint32_t REPLY_EVERY_DEFAULT = 100;
 
 const uint32_t TEST_START_WARMUP_MSEC = 400;
 const uint32_t TEST_END_COOLDOWN_MSEC = 50;
+const uint64_t TEST_START_WARMUP_NUM = 8000;
+const uint64_t TEST_END_COOLDOWN_NUM = 1000;
 
 const uint32_t TEST_FIRST_CONNECTION_FIRST_PACKET_TTL_THRESHOLD_MSEC = 50;
 #define TEST_ANY_CONNECTION_FIRST_PACKET_TTL_THRESHOLD_MSEC (0.1)
@@ -126,36 +167,50 @@ const uint32_t TEST_FIRST_CONNECTION_FIRST_PACKET_TTL_THRESHOLD_MSEC = 50;
 #define DEFAULT_CLIENT_WORK_WITH_SRV_NUM 1
 
 #define DEFAULT_TEST_DURATION 1 /* [sec] */
-#define DEFAULT_MC_ADDR "0.0.0.0"
-#define DEFAULT_PORT 11111
+#define DEFAULT_TEST_NUMBER 0   /* [number of packets] */
+#define DEFAULT_PORT_STR "11111"
 #define DEFAULT_IP_MTU 1500
 #define DEFAULT_IP_PAYLOAD_SZ (DEFAULT_IP_MTU - 28)
+#define DEFAULT_CI_SIG_LEVEL 99
 #define DUMMY_PORT 57341
 #define MAX_ACTIVE_FD_NUM                                                                          \
     1024 /* maximum number of active connection to the single TCP addr:port                        \
             */
-#ifdef USING_VMA_EXTRA_API
+#ifdef USING_VMA_EXTRA_API // For VMA socketxtreme Only
 #define MAX_VMA_COMPS 1024 /* maximum size for the VMA completions array for VMA Poll */
-#endif
+#endif // USING_VMA_EXTRA_API
 
 #ifndef MAX_PATH_LENGTH
 #define MAX_PATH_LENGTH 1024
 #endif
-#define MAX_MCFILE_LINE_LENGTH 41 /* sizeof("U:255.255.255.255:11111:255.255.255.255\0") */
+#define MAX_MCFILE_LINE_LENGTH 1024 /* big enough to store IPv6 addresses and hostnames */
 
 #define IP_PORT_FORMAT_REG_EXP                                                                     \
-    "^([UuTt]:)*([a-zA-Z0-9\\.\\-]+):(6553[0-5]|655[0-2][0-9]|65[0-4][0-9]{2}|6[0-4][0-9]{3}|[0-"  \
-    "5]?[0-9]{1,4})(:[a-zA-Z0-9\\.\\-]+)?[\r\n]"
-/*
-#define IP_PORT_FORMAT_REG_EXP		"^([UuTt]:)*((25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\\.){3}"\
-                    "(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?):"\
-                    "(6553[0-5]|655[0-2][0-9]|65[0-4][0-9]{2}|6[0-4][0-9]{3}|[0-5]?[0-9]{1,4})\n"
-*/
+    "^([UuTt]:)?"                                                                                  \
+    "([a-zA-Z0-9\\.\\-]+|\\[([a-fA-F0-9.:]+(%[0-9a-zA-z])?)\\])"                                   \
+    ":(6553[0-5]|655[0-2][0-9]|65[0-4][0-9]{2}|6[0-4][0-9]{3}|[0-5]?[0-9]{1,4})"                   \
+    "(:([a-zA-Z0-9\\.\\-]+|\\[([a-fA-F0-9.:]+)\\]))?[\r\n]"
+
+#ifdef __windows__
+#define UNIX_DOMAIN_SOCKET_FORMAT_REG_EXP                                                          \
+        "^[UuTt]:([A-Za-z]:[\\\\/].*)[\r\n]*"
+#define RESOLVE_ADDR_FORMAT_SOCKET                                                                 \
+        "[A-Za-z]:[\\\\/].*"
+#elif __linux__
+#define UNIX_DOMAIN_SOCKET_FORMAT_REG_EXP                                                          \
+        "^[UuTt]:(/.+)[\r\n]"
+#define RESOLVE_ADDR_FORMAT_SOCKET                                                                 \
+        "/.+"
+#endif
+
 #define PRINT_PROTOCOL(type)                                                                       \
     ((type) == SOCK_DGRAM ? "UDP" : ((type) == SOCK_STREAM ? "TCP" : "<?>"))
+#define PRINT_SOCKET_TYPE(type)                                                                     \
+    ((type) == SOCK_DGRAM ? "SOCK_DGRAM" : ((type) == SOCK_STREAM ? "SOCK_STREAM" : "<?>"))
 
 #define MAX_ARGV_SIZE 256
 #define MAX_DURATION 36000000
+#define MAX_PACKET_NUMBER 100000000
 extern const int MAX_FDS_NUM;
 #define SOCK_BUFF_DEFAULT_SIZE 0
 #define DEFAULT_SELECT_TIMEOUT_MSEC 10
@@ -185,8 +240,8 @@ enum {
     OPT_NONBLOCKED,               // 9
     OPT_DONTWARMUP,               // 10
     OPT_PREWARMUPWAIT,            // 11
-    OPT_VMARXFILTERCB,            // 12
-    OPT_VMAZCOPYREAD,             // 13
+    OPT_RXFILTERCB,               // 12
+    OPT_ZCOPYREAD,                // 13
     OPT_SOCKETXTREME,             // 14
     OPT_MC_LOOPBACK_ENABLE,       // 15
     OPT_CLIENT_WORK_WITH_SRV_NUM, // 16
@@ -210,21 +265,27 @@ enum {
     OPT_RECV_LOOPING,             // 34
     OPT_OUTPUT_PRECISION,         // 35
     OPT_CLIENTPORT,               // 36
-    OPT_CLIENTIP,                 // 37
+    OPT_CLIENTADDR,               // 37
     OPT_TOS,                      // 38
     OPT_LLS,                      // 39
     OPT_MC_SOURCE_IP,             // 40
     OPT_DUMMY_SEND,               // 41
     OPT_RATE_LIMIT,               // 42
     OPT_UC_REUSEADDR,             // 43
-    OPT_FULL_RTT                  // 44
+    OPT_FULL_RTT,                 // 44
+    OPT_CI_SIG_LVL,               // 45
+    OPT_HISTOGRAM,                // 46
+    OPT_LOAD_XLIO,                // 47
+#if defined(DEFINED_TLS)
+    OPT_TLS
+#endif /* DEFINED_TLS */
 };
 
 static const char *const round_trip_str[] = { "latency", "rtt" };
 
 #define MODULE_NAME "sockperf"
 #define MODULE_COPYRIGHT                                                                           \
-    "Copyright (C) 2011 Mellanox Technologies Ltd."                                                \
+    "Copyright (C) 2011-2022 Mellanox Technologies Ltd."                                           \
     "\nSockPerf is open source software, see http://github.com/mellanox/sockperf"
 #define log_msg(log_fmt, ...) printf(MODULE_NAME ": " log_fmt "\n", ##__VA_ARGS__)
 #define log_msg_file(file, log_fmt, ...) fprintf(file, MODULE_NAME ": " log_fmt "\n", ##__VA_ARGS__)
@@ -427,25 +488,23 @@ extern TicksTime g_cycleStartTime;
 
 extern debug_level_t g_debug_level;
 
-#ifdef USING_VMA_EXTRA_API
-
+#if defined(USING_VMA_EXTRA_API) || defined(USING_XLIO_EXTRA_API)
+#ifdef USING_VMA_EXTRA_API // VMA
 extern struct vma_buff_t *g_vma_buff;
 extern struct vma_completion_t *g_vma_comps;
-
+#endif // USING_VMA_EXTRA_API
 class ZeroCopyData {
 public:
     ZeroCopyData();
     void allocate();
     ~ZeroCopyData();
     unsigned char *m_pkt_buf;
-    struct vma_packets_t *m_pkts;
-    unsigned int m_pkt_index;
-    unsigned int m_pkt_offset;
+    void *m_pkts;
 };
 // map from fd to zeroCopyData
 typedef std::map<int, ZeroCopyData *> zeroCopyMap;
 extern zeroCopyMap g_zeroCopyData;
-#endif
+#endif // USING_VMA_EXTRA_API || USING_XLIO_EXTRA_API
 
 class Message;
 
@@ -461,38 +520,53 @@ typedef struct spike {
     int next;
 } spike;
 
-typedef struct port_and_type {
-    int sock_type; /**< SOCK_STREAM (tcp), SOCK_DGRAM (udp), SOCK_RAW (ip) */
-    in_port_t port;
-} port_type;
+struct SocketRecvData {
+    uint8_t *buf = nullptr;         // buffer for input messages (double size is reserved)
+    int max_size = 0;               // maximum message size
+    uint8_t *cur_addr = nullptr;    // start of current message (may point outside buf)
+    int cur_offset = 0;             // number of available message bytes
+    int cur_size = 0;               // maximum number of bytes for the next chunk
+};
+
+// big enough to store sockaddr_in and sockaddr_in6
+struct sockaddr_store_t {
+    union {
+        sa_family_t ss_family;
+        sockaddr_in addr4;
+        sockaddr_in6 addr6;
+        sockaddr_un addr_un;
+    };
+};
 
 /**
  * @struct fds_data
  * @brief Socket related info
  */
 
-typedef struct fds_data {
-    struct sockaddr_in server_addr; /**< server address information */
-    int is_multicast;               /**< if this socket is multicast */
-    int sock_type;                  /**< SOCK_STREAM (tcp), SOCK_DGRAM (udp), SOCK_RAW (ip) */
-    int next_fd;
-    int active_fd_count; /**< number of active connections (by default 1-for UDP; 0-for TCP) */
-    int *
-    active_fd_list; /**< list of fd related active connections (UDP has the same fd by default) */
-    struct sockaddr_in *memberships_addr; /**< more servers on the same socket information */
-    struct in_addr mc_source_ip_addr;     /**< message source ip for multicast packet filtering */
-    int memberships_size;
-    struct {
-        uint8_t *buf;
-        int max_size;
-        uint8_t *cur_addr;
-        int cur_offset;
-        int cur_size;
-    } recv;
-#ifdef USING_VMA_EXTRA_API
-    Message *p_msg;
-#endif
-} fds_data;
+struct fds_data {
+    struct sockaddr_store_t server_addr; /**< server address information */
+    socklen_t server_addr_len = 0;  /**< server address length */
+    int is_multicast = 0;           /**< if this socket is multicast */
+    int sock_type = 0;              /**< SOCK_STREAM (tcp), SOCK_DGRAM (udp), SOCK_RAW (ip) */
+    int next_fd = 0;
+    int active_fd_count = 0;        /**< number of active connections (by default 1-for UDP; 0-for TCP) */
+    int *active_fd_list = nullptr;  /**< list of fd related active connections (UDP has the same fd by default) */
+    struct sockaddr_store_t *memberships_addr = nullptr; /**< more servers on the same socket information */
+    IPAddress mc_source_ip_addr;    /**< message source ip for multicast packet filtering */
+    int memberships_size = 0;
+    struct SocketRecvData recv;
+#ifdef USING_VMA_EXTRA_API // VMA callback-extra-api Only
+    Message *p_msg = nullptr; // For VMA callback API.
+#endif // USING_VMA_EXTRA_API
+#if defined(DEFINED_TLS)
+    void *tls_handle = nullptr;
+#endif /* DEFINED_TLS */
+
+    fds_data()
+    {
+        memset(&server_addr, 0, sizeof(server_addr));
+    }
+};
 
 /**
  * @struct handler_info
@@ -508,7 +582,7 @@ typedef struct handler_info {
 typedef struct clt_session_info {
     uint64_t seq_num;
     uint64_t total_drops;
-    sockaddr_in addr;
+    sockaddr_store_t addr;
     bool started;
 } clt_session_info_t;
 
@@ -516,86 +590,82 @@ typedef struct clt_session_info {
 // std:: string, and std::wstring. For any other type, we need to write a
 // hash/equal_to functions, by ourself.
 namespace std {
-#if !defined(WIN32) && !defined(__FreeBSD__)
-namespace tr1 {
-#endif
-template <> struct hash<struct sockaddr_in> : public std::unary_function<struct sockaddr_in, int> {
-    int operator()(struct sockaddr_in const &key) const {
+template <> struct hash<struct sockaddr_store_t> : public std::unary_function<struct sockaddr_store_t, int> {
+    int operator()(struct sockaddr_store_t const &key) const {
         // XOR "a.b" part of "a.b.c.d" address with 16bit port; leave "c.d" part untouched for
         // maximum hashing
-        return key.sin_addr.s_addr ^ key.sin_port;
+        switch (key.ss_family) {
+        case AF_INET: {
+            const sockaddr_in &k = reinterpret_cast<const sockaddr_in &>(key);
+            return k.sin_addr.s_addr ^ (k.sin_port << 16);
+        }
+        case AF_INET6: {
+            const sockaddr_in6 &k = reinterpret_cast<const sockaddr_in6 &>(key);
+            const uint32_t *addr = reinterpret_cast<const uint32_t *>(&k.sin6_addr);
+            return addr[0] ^ addr[1] ^ addr[2] ^ addr[3] ^ (k.sin6_port << 16);
+        }
+        default:
+            return 0;
+        }
     }
 };
 
 template <>
-struct hash<struct port_and_type> : public std::unary_function<struct port_and_type, int> {
-    int operator()(struct port_and_type const &key) const {
-        // XOR "a.b" part of "a.b.c.d" address with 16bit port; leave "c.d" part untouched for
-        // maximum hashing
-        return key.sock_type ^ key.port;
+struct equal_to<struct sockaddr_store_t> :
+        public std::binary_function<struct sockaddr_store_t,
+                struct sockaddr_store_t, bool> {
+    bool operator()(struct sockaddr_store_t const &key1, struct sockaddr_store_t const &key2) const {
+        if (key1.ss_family != key2.ss_family) {
+            return false;
+        }
+        switch (key1.ss_family) {
+        case AF_INET: {
+            const sockaddr_in &k1 = reinterpret_cast<const sockaddr_in &>(key1);
+            const sockaddr_in &k2 = reinterpret_cast<const sockaddr_in &>(key2);
+            return k1.sin_port == k2.sin_port &&
+                k1.sin_addr.s_addr == k2.sin_addr.s_addr;
+        }
+        case AF_INET6: {
+            const sockaddr_in6 &k1 = reinterpret_cast<const sockaddr_in6 &>(key1);
+            const sockaddr_in6 &k2 = reinterpret_cast<const sockaddr_in6 &>(key2);
+            return k1.sin6_port == k2.sin6_port &&
+                IN6_ARE_ADDR_EQUAL(&k1.sin6_addr, &k2.sin6_addr);
+        }
+        }
+        return true;
     }
 };
+} // namespace std
 
-template <> struct hash<struct in_addr> : public std::unary_function<struct in_addr, int> {
-    int operator()(struct in_addr const &key) const { return key.s_addr & 0xFF; }
-};
-#if !defined(WIN32) && !defined(__FreeBSD__)
-} // closes namespace tr1
-#endif
-template <>
-struct equal_to<struct sockaddr_in> : public std::binary_function<struct sockaddr_in,
-                                                                  struct sockaddr_in, bool> {
-    bool operator()(struct sockaddr_in const &key1, struct sockaddr_in const &key2) const {
-        return key1.sin_port == key2.sin_port && key1.sin_addr.s_addr == key2.sin_addr.s_addr;
-    }
-};
-
-template <>
-struct equal_to<struct port_and_type> : public std::binary_function<struct port_and_type,
-                                                                    struct port_and_type, bool> {
-    bool operator()(struct port_and_type const &key1, struct port_and_type const &key2) const {
-        return key1.sock_type == key2.sock_type && key1.port == key2.port;
-    }
-};
-
-template <>
-struct equal_to<struct in_addr> : public std::binary_function<struct in_addr, struct in_addr,
-                                                              bool> {
-    bool operator()(struct in_addr const &key1, struct in_addr const &key2) const {
-        return key1.s_addr == key2.s_addr;
-    }
-};
-}
-
-#ifdef USING_VMA_EXTRA_API
+#ifdef USING_VMA_EXTRA_API // VMA socketxtreme-extra-api Only
 struct vma_ring_comps {
     vma_completion_t vma_comp_list[MAX_VMA_COMPS];
     int vma_comp_list_size;
     bool is_freed;
 };
-#endif
+#endif // USING_VMA_EXTRA_API
 
-#ifndef __FreeBSD__
-typedef std::tr1::unordered_map<struct sockaddr_in, clt_session_info_t> seq_num_map;
-typedef std::tr1::unordered_map<struct in_addr, size_t> addr_to_id;
-#ifdef USING_VMA_EXTRA_API
-typedef std::tr1::unordered_map<int, struct vma_ring_comps *> rings_vma_comps_map;
-#endif
-#else
-typedef std::unordered_map<struct sockaddr_in, clt_session_info_t> seq_num_map;
-typedef std::unordered_map<struct in_addr, size_t> addr_to_id;
-#ifdef USING_VMA_EXTRA_API
+typedef std::unordered_map<struct sockaddr_store_t, clt_session_info_t> seq_num_map;
+typedef std::unordered_map<IPAddress, size_t> addr_to_id;
+#ifdef USING_VMA_EXTRA_API // VMA socketxtreme-extra-api Only
 typedef std::unordered_map<int, struct vma_ring_comps *> rings_vma_comps_map;
-#endif
-#endif
+#endif // USING_VMA_EXTRA_API
 
 extern fds_data **g_fds_array;
 extern int IGMP_MAX_MEMBERSHIPS;
 
-#ifdef USING_VMA_EXTRA_API
+#ifdef USING_VMA_EXTRA_API // VMA
 typedef std::queue<int> vma_comps_queue;
 extern struct vma_api_t *g_vma_api;
-#endif
+#else
+extern void *g_vma_api; // Dummy variable
+#endif // USING_VMA_EXTRA_API
+
+#ifdef USING_XLIO_EXTRA_API // XLIO
+extern struct xlio_api_t *g_xlio_api;
+#else
+extern void *g_xlio_api; // Dummy variable
+#endif // USING_XLIO_EXTRA_API
 
 typedef enum {
     MODE_CLIENT = 0,
@@ -603,77 +673,109 @@ typedef enum {
     MODE_BRIDGE
 } work_mode_t;
 
+typedef enum {
+    TIME_BASED = 1,
+    NUMBER_BASED
+} measurement_mode_t;
+
 typedef enum { // must be coordinated with s_fds_handle_desc in common.cpp
     RECVFROM = 0,
     RECVFROMMUX,
     SELECT,
-#ifndef WIN32
+#ifndef __windows__
     POLL,
     EPOLL,
-    SOCKETXTREME,
 #endif
+    SOCKETXTREME,
     FD_HANDLE_MAX } fd_block_handler_t;
 
 struct user_params_t {
-    work_mode_t mode; // either  client or server
-    struct in_addr rx_mc_if_addr;
-    struct in_addr tx_mc_if_addr;
-    struct in_addr mc_source_ip_addr;
-    int msg_size;
-    int msg_size_range;
-    int sec_test_duration;
-    bool data_integrity;
-    fd_block_handler_t fd_handler_type;
-    unsigned int packetrate_stats_print_ratio;
-    unsigned int burst_size;
-    bool packetrate_stats_print_details;
-    //	bool stream_mode; - use b_stream instead
-    int mthread_server;
-    struct timeval *select_timeout;
-    int sock_buff_size;
-    int threads_num;
+    work_mode_t mode = MODE_SERVER; // either client or server
+    measurement_mode_t measurement = TIME_BASED; // either time or number
+    int rx_mc_if_ix;
+    bool rx_mc_if_ix_specified;
+    int tx_mc_if_ix;
+    bool tx_mc_if_ix_specified;
+    IPAddress rx_mc_if_addr;
+    IPAddress tx_mc_if_addr;
+    IPAddress mc_source_ip_addr;
+    int msg_size = MIN_PAYLOAD_SIZE;
+    int msg_size_range = 0;
+    int sec_test_duration = DEFAULT_TEST_DURATION;
+    uint64_t number_test_target = DEFAULT_TEST_NUMBER;
+    bool data_integrity = false;
+    fd_block_handler_t fd_handler_type = RECVFROM;
+    unsigned int packetrate_stats_print_ratio = 0;
+    unsigned int burst_size = 1;
+    bool packetrate_stats_print_details = false;
+    int mthread_server = 0;
+    struct timeval *select_timeout = nullptr;
+    int sock_buff_size = SOCK_BUFF_DEFAULT_SIZE;
+    int threads_num = 1;
     char threads_affinity[MAX_ARGV_SIZE];
-    bool is_blocked;
-    bool do_warmup;
-    unsigned int pre_warmup_wait;
-    uint32_t warmup_msec;
-    bool is_vmarxfiltercb;
-    bool is_vmazcopyread;
+    bool is_rxfiltercb = false;
+    bool is_zcopyread = false;
+    bool is_blocked = true;
+    bool do_warmup = true;
+    unsigned int pre_warmup_wait = 0;
+    uint32_t cooldown_msec = 0;
+    uint32_t warmup_msec = 0;
+    uint64_t cooldown_num = 0;
+    uint64_t warmup_num = 0;
+    bool is_vmarxfiltercb = false;
+    bool is_vmazcopyread = false;
     TicksDuration cycleDuration;
-    bool mc_loop_disable;
-    bool uc_reuseaddr;
-    int client_work_with_srv_num;
-    bool b_server_reply_via_uc;
-    bool b_server_dont_reply;
-    bool b_server_detect_gaps;
-    uint32_t mps; // client side only
-    struct sockaddr_in client_bind_info;
-    uint32_t reply_every;    // client side only
-    bool b_client_ping_pong; // client side only
-    bool b_no_rdtsc;
+    bool mc_loop_disable = true;
+    bool uc_reuseaddr = false;
+    int client_work_with_srv_num = DEFAULT_CLIENT_WORK_WITH_SRV_NUM;
+    bool b_server_reply_via_uc = false;
+    bool b_server_dont_reply = false;
+    bool b_server_detect_gaps = false;
+    uint32_t mps = MPS_DEFAULT; // client side only
+    struct sockaddr_store_t client_bind_info;
+    socklen_t client_bind_info_len = 0;
+    uint32_t reply_every = REPLY_EVERY_DEFAULT;    // client side only
+    bool b_client_ping_pong = false; // client side only
+    bool b_no_rdtsc = false;
     char sender_affinity[MAX_ARGV_SIZE];
     char receiver_affinity[MAX_ARGV_SIZE];
-    FILE *fileFullLog;               // client side only
-    bool full_rtt;                   // client side only
-    bool giga_size;                  // client side only
-    bool increase_output_precision;  // client side only
-    bool b_stream;                   // client side only
-    PlaybackVector *pPlaybackVector; // client side only
-    struct sockaddr_in addr;
-    int sock_type;
-    bool tcp_nodelay;
-    bool is_nonblocked_send;
-    int mc_ttl;
-    int daemonize;
+    FILE *fileFullLog = NULL;                   // client side only
+    bool full_rtt = false;                      // client side only
+    bool giga_size = false;                     // client side only
+    bool increase_output_precision = false;     // client side only
+    bool b_stream = false;                      // client side only
+    PlaybackVector *pPlaybackVector = NULL;     // client side only
+    uint32_t ci_significance_level = DEFAULT_CI_SIG_LEVEL;// client side only
+    bool b_histogram;                           // client side only
+    uint32_t histogram_lower_range = 0;         // client side only
+    uint32_t histogram_upper_range = 2000000;   // client side only
+    uint32_t histogram_bin_size = 10;           // client side only
+    struct sockaddr_store_t addr;
+    socklen_t addr_len = 0;
+    int sock_type = SOCK_DGRAM;
+    bool tcp_nodelay = true;
+    bool is_nonblocked_send = false;
+    int mc_ttl = 2;
+    int daemonize = false;
     char feedfile_name[MAX_PATH_LENGTH];
-    bool withsock_accl;
-    int max_looping_over_recv;
-    int tos;
-    unsigned int lls_usecs;
-    bool lls_is_set;
-    uint32_t dummy_mps;                   // client side only
+    bool withsock_accl = false;
+    int max_looping_over_recv = 1;
+    int tos = 0x00;
+    unsigned int lls_usecs = 0;
+    bool lls_is_set = false;
+    uint32_t dummy_mps = 0;                   // client side only
     TicksDuration dummySendCycleDuration; // client side only
-    uint32_t rate_limit;
+    uint32_t rate_limit = 0;
+#if defined(DEFINED_TLS)
+    bool tls = false;
+#endif /* DEFINED_TLS */
+
+    user_params_t() {
+        memset(&client_bind_info, 0, sizeof(client_bind_info));
+        memset(&addr, 0, sizeof(addr));
+        memset(sender_affinity, 0, sizeof(sender_affinity));
+        memset(receiver_affinity, 0, sizeof(receiver_affinity));
+    }
 };
 
 struct mutable_params_t {};
@@ -683,8 +785,6 @@ class App {
 public:
     App(const struct user_params_t &_user_params, const struct mutable_params_t &_mutable_params)
         : m_const_params(_user_params), m_mutable_params(_mutable_params) {
-
-        TicksBase::init(m_const_params.b_no_rdtsc ? TicksBase::CLOCK : TicksBase::RDTSC);
     }
     ~App() {}
 
