@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011-2018 Mellanox Technologies Ltd.
+ * Copyright (c) 2011-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without modification,
@@ -28,6 +28,8 @@
 
 #ifndef IOHANDLERS_H_
 #define IOHANDLERS_H_
+
+#include "defs.h"
 #include "common.h"
 
 //==============================================================================
@@ -228,7 +230,7 @@ private:
     fd_set m_readfds, m_save_fds;
 };
 
-#ifndef WIN32
+#ifndef __windows__
 //==============================================================================
 class IoPoll : public IoHandler {
 public:
@@ -377,7 +379,7 @@ private:
 };
 #endif
 
-#ifdef USING_VMA_EXTRA_API
+#ifdef USING_VMA_EXTRA_API // VMA socketxtreme-extra-api Only
 //==============================================================================
 class IoSocketxtreme : public IoHandler {
 public:
@@ -468,6 +470,8 @@ public:
         } else if (g_vma_comps->events & VMA_SOCKETXTREME_PACKET) {
             g_vma_buff = g_vma_comps->packet.buff_lst;
             ifd = g_vma_comps->user_data;
+        } else if (g_vma_comps->events & (EPOLLIN | EPOLLERR | EPOLLHUP | EPOLLRDHUP)) {
+            ifd = g_vma_comps->user_data;
         } else {
             ifd = 0;
         }
@@ -489,6 +493,6 @@ private:
     rings_vma_comps_map m_rings_vma_comps_map;
     rings_vma_comps_map::iterator m_rings_vma_comps_map_itr;
 };
-#endif
-#endif
-#endif /* IOHANDLERS_H_ */
+#endif // USING_VMA_EXTRA_API
+#endif // !WIN32
+#endif // IOHANDLERS_H_
